package anynt.object;


/**
 * It extends an agent to represent a evaluable agent
 * 
 * @author Javier
 * @since 30/03/2011
 */
public class EvaluableAgent extends Agent
{
	// Constructors
	public EvaluableAgent(String name)
	{
		super(name);
	}

	// Object functions
	/**
	 * It returns the last obtained reward
	 */
	public double lastReward()
	{
		return getActualInteraction().getReward();
	}
	/**
	 * It returns the total reward obtained until a certain number of interactions
	 */
	public double totalReward(int interactions)
	{
		if (interactions < 0)
			throw new IllegalArgumentException();

		double reward = 0.0;
		// It calculates the number of interactions to consider
		int interactionsDone = interactionHistory.size() - 1;
		int maxInteraction = Math.min(interactions, interactionsDone);
		if (maxInteraction == 0)
			return reward;

		// Sum all the obtained rewards
		for (int index = 1; index <= maxInteraction; index++)
			reward += interactionHistory.get(index).getReward();

		// It returns the average of the obtained rewards
		return reward / maxInteraction;
	}
	/**
	 * It returns the total reward obtained
	 */
	public double totalReward()
	{
		double reward = 0.0;
		int interactions = interactionHistory.size() - 1;
		if (interactions == 0)
			return reward;

		// Sum all the obtained rewards
		for (int index = 1; index <= interactions; index++)
			reward += interactionHistory.get(index).getReward();

		// It returns the average of the obtained rewards
		return reward / interactions;
	}
}