package anynt.object.behaviour;


import java.util.ArrayList;
import java.util.Collection;

import anynt.object.Agent;
import anynt.space.Observation;


/**
 * It represents the behavior that will have the agent
 * 
 * @author Javier
 * @since 31/03/2011
 */
public abstract class AgentBehaviour
{
	// Object attributes
	/**
	 * The agent that has this behavior
	 */
	protected Agent agent;
	/**
	 * Time elapsed that the agent has spent to realize his actions on each interaction
	 */
	protected long elapsedTime;


	// Object overrides
	public abstract AgentBehaviour clone();

	// Treatment of object attributes
	public long getElapsedTime()
	{
		return elapsedTime;
	}
	public void resetElapsedTime()
	{
		elapsedTime = 0;
	}
	public void setAgent(Agent agent)
	{
		this.agent = agent;
	}

	// Object functions
	public void checkConsistency()
	{
		if (agent == null)
			throw new RuntimeException("A behavior agent is not defined");
	}
	/**
	 * Reset the agent behavior to his initial definition
	 */
	public void reset()
	{
		elapsedTime = 0;
	}
	/**
	 * Actualizes the behavior of the agent if it's necessary
	 */
	public void actualize()
	{
		
	}

	/**
	 * It returns the next action of the agent
	 */
	public abstract int nextAction(Observation observation);
	/**
	 * It returns the actions that the agent is able to do in this cell
	 */
	protected Collection<Integer> availableActions()
	{
		ArrayList<Integer> actions = new ArrayList<Integer>();

		Collection<Integer> cellActions = agent.getLocation().getAvailableActions();
		for (Integer action : agent.getKnownActions())
			if (cellActions.contains(action))
				actions.add(action);

		return actions;
	}
}