package anynt.object.behaviour;


import java.util.ArrayList;
import java.util.Random;

import distribution.IDistribution;
import distribution.UniformDistribution;

import anynt.object.RewardsGeneratorAgent;
import anynt.space.Cell;
import anynt.space.Observation;
import anynt.space.Space;


/**
 * It extends the behavior of an agent to represent a behavior trying to follow to Good and avoids Evil
 * 
 * @author Javier
 * @since 31/03/2011
 */
public class TrivialFollowerBehaviour extends AgentBehaviour
{
	// Object attributes
	/**
	 * Good agent that is followed
	 */
	private final RewardsGeneratorAgent goodAgent;
	/**
	 * Evil agent that is avoided
	 */
	private final RewardsGeneratorAgent evilAgent;


	// Constructors
	public TrivialFollowerBehaviour(RewardsGeneratorAgent goodAgent, RewardsGeneratorAgent evilAgent)
	{
		this.goodAgent = goodAgent;
		this.evilAgent = evilAgent;
	}

	// Object overrides
	public TrivialFollowerBehaviour clone()
	{
		return new TrivialFollowerBehaviour(goodAgent, evilAgent);
	}

	// AgentBehaviour overrides
	public int nextAction(Observation observation)
	{
		// It actualizes the time spend to realize the action
		long maximumTime = agent.getMaximumTimeToAct();
		elapsedTime += new Random().nextLong() % (maximumTime + 1);

		// It retrieves the position of the agent in the observation
		int locationNumber = agent.getLocation().getNumber();
		Space space = (Space) observation.getObservation();
		Cell location = space.getCell(locationNumber);

		// Actions that can be made
		ArrayList<Integer> possibleActions = new ArrayList<Integer>();

		// Check all actions that can be realized
		for (int action : availableActions())
		{
			// Cell where the agent can be moved
			Cell adyacentCell = location.destinationCell(action);
			// If in the cell there is Good agent then this is the action to do
			if (adyacentCell.getAgents().contains(goodAgent))
				return action;
			// If in the cell there is not Evil agent then this action can be made
			if (!adyacentCell.getAgents().contains(evilAgent))
				possibleActions.add(action);
		}
		// If it's not possible to make an action without encounter Evil rest in the cell
		if (possibleActions.size() == 0)
			possibleActions.add(0);

		// It return one of the possible actions randomly
		IDistribution uniformDistribution = new UniformDistribution();
		return possibleActions.get(uniformDistribution.getValue(possibleActions.size()));
	}
}