package anynt.object.moveController;


import anynt.object.Agent;
import anynt.object.behaviour.AgentBehaviour;
import anynt.space.Observation;


/**
 * It implements the interface of a move controller to realize always a certain number of actions
 * 
 * @author Javier
 * @since 31/03/2011
 */
public class NumberOfMovementsMoveController implements IAgentMoveController
{
	// Object attributes
	/**
	 * Number of movements that the agent makes on each interaction
	 */
	private int numberOfMovements;


	// Constructors
	public NumberOfMovementsMoveController()
	{
		this(1);
	}
	public NumberOfMovementsMoveController(int number)
	{
		numberOfMovements = number;
	}

	// IAgentMoveController overrides
	/**
	 * It returns the actions that the agent will realize in the interaction
	 */
	public int[] makeAction(Agent agent, Observation observation)
	{
		int[] actions = new int[numberOfMovements];

		// Restarts the elapsed timer in the interaction
		AgentBehaviour behaviour = agent.getBehaviour();
		behaviour.resetElapsedTime();
		for (int movement = 0; movement < numberOfMovements; movement++)
			actions[movement] = behaviour.nextAction(observation);

		return actions;
	}

	// Treatment of object attributes
	public int getNumberOfMovements()
	{
		return numberOfMovements;
	}
	public void setNumberOfMovements(int number)
	{
		numberOfMovements = number;
	}
}