package anynt.object.moveController;


import java.util.Collection;

import anynt.object.Agent;
import anynt.object.behaviour.AgentBehaviour;
import anynt.object.behaviour.PatternBehaviour;
import anynt.space.Cell;
import anynt.space.Observation;


/**
 * Implementa la interfaz de un Controlador de movimiento para tratar de seguir un patrn
 * 
 * @author Javier
 * @since 31/03/2011
 */
public class PatternMoveController implements IAgentMoveController
{
	// IAgentMoveController overrides
	/**
	 * It returns the actions that the agent will realize in the interaction
	 */
	public int[] makeAction(Agent agent, Observation observation)
	{
		// It restarts the elapsed timer in the interaction
		AgentBehaviour behaviour = agent.getBehaviour();
		behaviour.resetElapsedTime();

		Cell agentCell = agent.getLocation();

		// Maximum number of actions that will be made
		int maxMovements = ((PatternBehaviour) behaviour).patternLenght();
		int[] actions = new int[maxMovements];
		for (int movement = 0; movement < maxMovements; movement++)
		{
			// It picks the action that will be made
			int action = behaviour.nextAction(observation);

			// If the action is not available, consume it and finish
			Collection<Integer> cellActions = agentCell.getAvailableActions();
			if (!cellActions.contains(action))
				break;

			// If the action is available store it and continue
			actions[movement] = action;
			agentCell = agentCell.destinationCell(action);
		}

		return actions;
	}
}