package anynt.test;


import interfaces.HumanAGITestInterface;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import anynt.environment.LambdaEnvironment;
import anynt.environment.AGILambdaEnvironment;
import anynt.environment.settings.NoShareRewards;
import anynt.environment.settings.RewardsProvider;
import anynt.exercise.Exercise;
import anynt.exercise.settings.InteractionType;
import anynt.exercise.settings.NumberOfInteractionsInteractionType;
import anynt.exercise.settings.StepWaiting;
import anynt.exercise.settings.TimeDelayWaiting;
import anynt.object.Agent;
import anynt.object.EvaluableAgent;
import anynt.object.RewardsGeneratorAgent;
import anynt.object.behaviour.AgentBehaviour;
import anynt.object.behaviour.HumanBehaviour;
import anynt.object.behaviour.PatternBehaviour;
import anynt.object.behaviour.QLearningBehaviour;
import anynt.object.moveController.IAgentMoveController;
import anynt.object.moveController.NumberOfMovementsMoveController;
import anynt.space.Space;


/**
 * It implements the test that is used for the AGI conference
 * 
 * @author Javier
 * @since 11/04/2011
 */
public class AGITest extends Test
{
	private static int NUMBER_OF_EXERCISES = 7;

	public static Test getTest()
	{
		if (Test.instance == null)
			Test.instance = new AGITest();
		return Test.instance;
	}


	// Object attributes
	private HumanAGITestInterface frame;

	/**
	 * Exercises that has been realized
	 */
	private ArrayList<Exercise> exercises;
	/**
	 * Moment when the test starts
	 */
	private long startTime;
	/**
	 * Momento when the test overs
	 */
	private long endTime;
	/**
	 * Name of the subject that realizes the test
	 */
	private String nombreSujeto;


	// Constructors
	private AGITest()
	{
		super();

		exercises = new ArrayList<Exercise>();
	}

	// Test overrides
	protected synchronized void prepareExercise()
	{
		exercise = new Exercise();
		frame.launchExercise();

		int numberOfInteractions = (exerciseNumber + 1) * 10;
		LambdaEnvironment environment = new AGILambdaEnvironment();

		Space space = new Space();
		space.setCellsBounds(exerciseNumber + 2, exerciseNumber + 2);
		space.setActionsBounds(2, 9);

		do
			space.generateSpace();
		while (!space.isStrongConnected());
		environment.setSpace(space);

		RewardsGeneratorAgent goodAgent = environment.getGoodAgent();
		RewardsGeneratorAgent evilAgent = environment.getEvilAgent();
		IAgentMoveController rewardsGeneratorAgentMoveController = new NumberOfMovementsMoveController(1);//new PatternMoveController();
		AgentBehaviour goodAgentBehaviour = new PatternBehaviour(PatternBehaviour.generatePattern(environment.getActions(), (numberOfInteractions / 10) + 1));
		AgentBehaviour evilAgentBehaviour = goodAgentBehaviour.clone();
		evilAgentBehaviour.setAgent(evilAgent);

		goodAgent.setKnownActions(environment.getActions());
		goodAgent.setMaximumObservationsSaved(1);
		goodAgent.setMoveController(rewardsGeneratorAgentMoveController);
		goodAgent.setBehaviour(goodAgentBehaviour);
		evilAgent.setKnownActions(environment.getActions());
		evilAgent.setMaximumObservationsSaved(1);
		evilAgent.setMoveController(rewardsGeneratorAgentMoveController);
		evilAgent.setBehaviour(evilAgentBehaviour);

		evaluatedAgent = new EvaluableAgent("Subject");
		IAgentMoveController evaluableAgentMoveController = new NumberOfMovementsMoveController();
		evaluatedAgent.setKnownActions(environment.getActions());
		evaluatedAgent.setMoveController(evaluableAgentMoveController);
		evaluatedAgent.setMaximumObservationsSaved(5);
		evaluatedAgent.setMaximumTimeToAct(5);
		evaluatedAgent.setBehaviour(new HumanBehaviour());

		environment.addAgent(evaluatedAgent);

		EvaluableAgent qLearning = new EvaluableAgent("QLearning");
		IAgentMoveController qLearningAgentMoveController = new NumberOfMovementsMoveController();
		qLearning.setKnownActions(environment.getActions());
		qLearning.setMoveController(qLearningAgentMoveController);
		qLearning.setMaximumObservationsSaved(5);
		qLearning.setMaximumTimeToAct(5);
		ArrayList<Agent> agentsForQLearning = new ArrayList<Agent>();
		agentsForQLearning.add(environment.getGoodAgent());
		agentsForQLearning.add(environment.getEvilAgent());
		agentsForQLearning.add(qLearning);
		qLearning.setBehaviour(new QLearningBehaviour(agentsForQLearning, 0.05, 0.0, 0.35, 2.0));
		environment.addAgent(qLearning);

		environment.setInteractionsToRelocateRewarders(0);
		InteractionType interactionType = new NumberOfInteractionsInteractionType(numberOfInteractions);
		exercise.setInteractionType(interactionType);
		environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(false);
		environment.setDecreaseFactor(Integer.MAX_VALUE);
		environment.setEvaluableAgentConsumeRewards(true);
		RewardsProvider rewardsSharing = new NoShareRewards();
		environment.setRewardsProvider(rewardsSharing);
		StepWaiting stepWaiting = new TimeDelayWaiting(0);
		exercise.setStepWaiting(stepWaiting);

		exercise.setEvaluableAgent(evaluatedAgent);
		exercise.setEnvironment(environment);

		try
		{
			wait();
		}
		catch (Exception e)
		{
			
		}

	}
	protected void finishExercise()
	{
		super.finishExercise();

		exercises.add(exercise);
	}
	protected void startTest()
	{
		super.startTest();

		startTime = System.currentTimeMillis();
	}
	protected void finishTest()
	{
		endTime = System.currentTimeMillis();
		super.finishTest();
		frame.showResults();

		long timeElapsed = getTakenTime();

		String file = nombreSujeto + "-" + timeElapsed + "-" + new SimpleDateFormat("HH.mm.ss").format(new Date()) + ".csv";
		for (Exercise exercise : exercises)
			files.ExperimentsFile.WriteExperiment(exercise, file);
	}
	protected boolean isTestFinished()
	{
		return exerciseNumber == NUMBER_OF_EXERCISES + 1;
	}

	// Treatment of object attributes
	public long getTakenTime()
	{
		return (endTime - startTime) / 1000;
	}
	public void setSubjectName(String name)
	{
		nombreSujeto = name;
	}
	public void setFrame(HumanAGITestInterface frame)
	{
		this.frame = frame;
	}
}