package anynt.test;


import anynt.exercise.Exercise;
import anynt.object.EvaluableAgent;


/**
 * It represent to the interface of a test
 * 
 * @author Javier
 * @since 11/04/2011
 */
public abstract class Test implements Runnable
{
	// Class attributes
	protected static Test instance;


	// Class functions
	public static Test getTest()
	{
		return Test.instance;
	}


	// Object attributes
	/**
	 * Actual exercise
	 */
	protected Exercise exercise;
	/**
	 * Agent that is evaluating
	 */
	protected EvaluableAgent evaluatedAgent;
	/**
	 * Average reward obtained so far
	 */
	private double averageReward;

	/**
	 * Number of exercise
	 */
	protected int exerciseNumber;


	// Runnable overrides
	public void run()
	{
		runTest();
	}

	// Treatment of object attributes
	public Exercise getExercise()
	{
		return exercise;
	}
	public void setExercise(Exercise exercise)
	{
		this.exercise = exercise;
	}
	public void setEvaluatedAgent(EvaluableAgent evaluableAgent)
	{
		this.evaluatedAgent = evaluableAgent;
	}
	public double getAverageReward()
	{
		return averageReward;
	}
	public int getExerciseNumber()
	{
		return exerciseNumber;
	}

	// Object functions
	public void start()
	{
		new Thread(this, "Test").start();
	}
	/**
	 * Executes the test
	 */
	public synchronized void runTest()
	{
		try
		{
			// It prepares the test
			startTest();
			do
			{
				// It prepares the exercise
				prepareExercise();
				// It launch the exercise
				launchExercise();

				// It finishes the exercise
				finishExercise();
				exerciseNumber++;
			}
			while (!isTestFinished());
			// It finishes the test
			finishTest();
		}
		catch (Exception e)
		{
			System.out.println(e.getStackTrace());
			throw new RuntimeException("Error during the test");
		}
	}
	protected void startTest()
	{
		averageReward = 0.0;
		exerciseNumber = 1;
	}
	protected abstract void prepareExercise();
	private void launchExercise()
	{
		exercise.start();
		try
		{
			exercise.join();
		}
		catch (InterruptedException e)
		{
			
		}
	}
	protected void finishExercise()
	{
		double reward = evaluatedAgent.totalReward();

		averageReward = (averageReward * (exerciseNumber - 1) + reward) / exerciseNumber;

		exercise.unregisterObservers();
	}
	protected abstract boolean isTestFinished();
	protected void finishTest()
	{
		
	}
}