package anynt.testEnvironments;


import anynt.environment.LambdaEnvironment;
import anynt.object.Agent;
import anynt.object.Interaction;
import anynt.object.Objeto;
import anynt.space.Cell;
import anynt.space.Connection;


public class StarsEnvironment extends LambdaEnvironment
{
	private int width;
	private int height;


	public StarsEnvironment(int width, int height) throws Exception
	{
		if (width < 3 || height < 3)
			throw new Exception("Error in size of environment: Minimum 3x3");

		actions = new int[5];
		actions[0] = 0;
		actions[1] = 1;
		actions[2] = 2;
		actions[3] = 3;
		actions[4] = 4;

		this.width = width;
		this.height = height;
	}


	public void describeEnvironment()
	{
		Cell[] cells = new Cell[width * height + 1];

		// Create cells
		cells[0] = new Cell(0);
		space.addVertex(cells[0]);
		for (int index = 1; index < cells.length; index++)
		{
			cells[index] = new Cell(index);
			Objeto object = new Objeto("Star");
			object.setLocation(cells[index]);
			cells[index].addObject(object);
			space.addVertex(cells[index]);
			Connection connection = new Connection(cells[index], cells[index], 0);
			space.addEdge(connection);
		}


		// Create the actions in the 4 corners
		space.addEdge(new Connection(cells[1], cells[2], 1));
		space.addEdge(new Connection(cells[1], cells[width + 1], 2));

		space.addEdge(new Connection(cells[width], cells[width - 1], 3));
		space.addEdge(new Connection(cells[width], cells[width * 2], 2));

		space.addEdge(new Connection(cells[width * (height - 1) + 1], cells[width * (height - 2) + 1], 4));
		space.addEdge(new Connection(cells[width * (height - 1) + 1], cells[width * (height - 1) + 2], 1));

		space.addEdge(new Connection(cells[width * height], cells[width * height - 1], 3));
		space.addEdge(new Connection(cells[width * height], cells[width * (height - 1)], 4));

		// Create the actions in the bottom side
		for (int index = 2; index <= width - 1; index++)
		{
			space.addEdge(new Connection(cells[index], cells[index - 1], 3));
			space.addEdge(new Connection(cells[index], cells[index + width], 2));
			space.addEdge(new Connection(cells[index], cells[index + 1], 1));
		}
		// Create the actions in the right side
		for (int index = width * 2; index <= width * (height - 1); index += width)
		{
			space.addEdge(new Connection(cells[index], cells[index - width], 4));
			space.addEdge(new Connection(cells[index], cells[index - 1], 3));
			space.addEdge(new Connection(cells[index], cells[index + width], 2));
		}
		// Create the actions in the left side
		for (int index = 1 + width; index <= 1 + (height - 2) * width; index += width)
		{
			space.addEdge(new Connection(cells[index], cells[index - width], 4));
			space.addEdge(new Connection(cells[index], cells[index + 1], 1));
			space.addEdge(new Connection(cells[index], cells[index + width], 2));
		}
		// Create the actions in the top side
		for (int index = 1 + (height - 1) * width + 1; index <= height * width - 1; index++)
		{
			space.addEdge(new Connection(cells[index], cells[index - 1], 3));
			space.addEdge(new Connection(cells[index], cells[index - width], 4));
			space.addEdge(new Connection(cells[index], cells[index + 1], 1));
		}

		// create the actions in the middle
		for (int row = 2; row < height; row++)
			for (int column = 2; column < width; column++)
			{
				space.addEdge(new Connection(cells[(row - 1) * width + column - 1], cells[(row - 1) * width + column - 2], 3));
				space.addEdge(new Connection(cells[(row - 1) * width + column - 1], cells[(row - 2) * width + column - 1], 4));
				space.addEdge(new Connection(cells[(row - 1) * width + column - 1], cells[(row - 1) * width + column], 1));
				space.addEdge(new Connection(cells[(row - 1) * width + column - 1], cells[row * width + column - 1], 2));
			}
	}


	public double evaluateAction(Agent agent, Interaction interaction)
	{
		double reward = 0.0;
		Cell cell = agent.getLocation();

		reward = -1.0;
		for (Objeto object : cell.getObjects())
			if (object.getName().equals("Star"))
			{
				reward = 1.0;
				cell.removeObject(object);
				break;
			}

		return reward;
	}
}