package graph;


import java.util.ArrayList;
import java.util.Collection;


/**
 * It represents a vertex in graph theory
 * 
 * @author Javier
 * @since 25/03/2001
 */
public class Vertex
{
	// Object attributes
	/**
	 * Number that identify the vertex
	 */
	private final int identifier;
	/**
	 * Edges that are connected with the vertex
	 */
	private final ArrayList<Edge> edges;


	// Constructors
	public Vertex(int identifier)
	{
		this.identifier = identifier;
		edges = new ArrayList<Edge>();
	}

	// Object overrides
	public Vertex clone()
	{
		return new Vertex(identifier);
	}
	public boolean equals(Object object)
	{
		if (!(object instanceof Vertex))
			return false;

		return identifier == ((Vertex) object).identifier;
	}
	public int hashCode()
	{
		return identifier;
	}

	// Treatment of object attributes
	public int getIdentifier()
	{
		return identifier;
	}
	public Collection<Edge> getEdges()
	{
		return edges;
	}
	public void addEdge(Edge edge)
	{
		edges.add(edge);
	}
	public void removeEdge(Edge edge)
	{
		edges.remove(edge);
	}

	// Object functions
	/**
	 * It returns all the adjacent vertices
	 */
	public Collection<Vertex> getConnected()
	{
		Collection<Vertex> adyacents = new ArrayList<Vertex>();

		for (Edge edge : edges)
			// It takes the vertex with which is connecting this vertex
			if (equals(edge.getEndVertex()))
				adyacents.add(edge.getStartVertex());
			else
				adyacents.add(edge.getEndVertex());

		return adyacents;
	}
	/**
	 * It returns all the adjacent vertices which can be accessed using some edge
	 */
	public Collection<Vertex> getAdyacents()
	{
		Collection<Vertex> adyacents = new ArrayList<Vertex>();

		for (Edge edge : edges)
			// Pick the vertex which can be accessed from this vertex
			if (equals(edge.getStartVertex()))
				adyacents.add(edge.getEndVertex());

		return adyacents;
	}
}