package human.AGI.object;


import human.AGI.HumanAGIExercisePanel;
import human.AGI.sound.SoundFile;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.Border;


import anynt.object.behaviour.HumanBehaviour;
import anynt.test.Test;


public class Cell extends JPanel implements MouseListener
{
	private static final long serialVersionUID = 1L;


	private static SoundFile SOUND = new SoundFile(Cell.class.getResource("/human/AGI/sound/click.wav"));
	private static final int AGENT_SEPARATION = 5;


	private Border borderNotConnected;
	private Border borderConnected;
	private Border borderAccesible;


	public void mouseClicked(MouseEvent arg0)
	{
		selectedCell();
	}
	public void mouseEntered(MouseEvent arg0)
	{
		candidateCell();
	}
	public void mouseExited(MouseEvent arg0)
	{
		notCandidateCell();
	}
	public void mousePressed(MouseEvent arg0)
	{
		
	}
	public void mouseReleased(MouseEvent arg0)
	{
		
	}


	private anynt.space.Cell cell;

	private int side;
	private int action;
	private boolean connected;
	private Color color;
	private Color darkColor;
	private ArrayList<Agent> agents;
	private boolean hasToBeSelected;


	public Cell(anynt.space.Cell cell)
	{
		this.cell = cell;

		side = 0;
		connected = false;
		color = null;
		agents = new ArrayList<Agent>();
		hasToBeSelected = false;

		setLayout(new FlowLayout(FlowLayout.LEADING, Cell.AGENT_SEPARATION, Cell.AGENT_SEPARATION));
		addMouseListener(this);

		borderNotConnected = BorderFactory.createCompoundBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(HumanAGIExercisePanel.BACKGROUND_COLOR, 3), BorderFactory.createLineBorder(HumanAGIExercisePanel.BACKGROUND_COLOR, 3)), BorderFactory.createLineBorder(Color.BLACK, 1));
		borderConnected = BorderFactory.createCompoundBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(HumanAGIExercisePanel.BACKGROUND_COLOR, 3), BorderFactory.createLineBorder(HumanAGIExercisePanel.BACKGROUND_COLOR, 3)), BorderFactory.createLineBorder(Color.BLACK, 3));
		borderAccesible = BorderFactory.createCompoundBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK, 3), BorderFactory.createLineBorder(HumanAGIExercisePanel.BACKGROUND_COLOR, 3)), BorderFactory.createLineBorder(Color.BLACK, 3));
	}


	public anynt.space.Cell getCell()
	{
		return cell;
	}
	public int getSide()
	{
		return side;
	}
	public void setSide(int side)
	{
		this.side = side;
		setPreferredSize(new Dimension(side, side));
	}
	public void setAction(int action)
	{
		this.action = action;
	}
	public void setColor(Color[] color)
	{
		this.color = color[0];
		darkColor = color[1];
		setBackground(color[0]);
	}
	public void setConnected(boolean connected)
	{
		this.connected = connected;

		if (hasToBeSelected)
			return;

		if (connected)
			setBorder(borderConnected);
		else
			setBorder(borderNotConnected);
	}
	public void clearAgents()
	{
		agents.clear();
		removeAll();
	}
	public void removeAgent(Agent agent)
	{
		agents.remove(agent);
	}
	public void addAgent(Agent agent)
	{
		agents.add(agent);
	}


	private void clear()
	{
		removeAll();
	}
	public void draw(/*Container parent*/)
	{
		//parent.add(this);

		invalidate();
		clear();

		for (Agent agent : agents)
			agent.draw(this);

		repaint();
		revalidate();
	}




	private void selectedCell()
	{
		if (!connected)
			return;

		Rewards rewards = ((HumanAGIExercisePanel) this.getParent().getParent()).getRewards();
		synchronized (rewards)
		{
			rewards.notify();
		}
		Cell.SOUND.play();
		hasToBeSelected = true;
		((HumanBehaviour) Test.getTest().getExercise().getEvaluableAgent().getBehaviour()).setAction(action);
	}
	private void candidateCell()
	{
		if (connected)
		{
			setBorder(borderAccesible);
			setBackground(darkColor);
		}
	}
	private void notCandidateCell()
	{
		if (connected)
			setBorder(borderConnected);
		setBackground(color);
		hasToBeSelected = false;
	}
}