package investigation;


import interfaces.InvestigationTestInterface;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;


@SuppressWarnings("serial")
public class InvestigationMenu extends JMenuBar
{
	public static boolean INITIAL_INTERFACE = true;
	public static boolean INITIAL_REWARDS = true;
	public static boolean INITIAL_AGENTS = true;


	private InvestigationTestInterface frame;

	private JMenu file;
	private JMenuItem generateEnvironment;
	private JMenuItem loadEnvironment;
	private JMenuItem saveEnvironment;
	private JMenuItem startExercise;
	private JMenuItem exit;

	private JMenu environment;
	private JMenuItem showInterface;
	private JMenuItem showRewards;
	private JMenuItem showAgents;


	public boolean isInterfaceShown()
	{
		return showInterface.isSelected();
	}
	public boolean areRewardsShown()
	{
		return showRewards.isSelected();
	}
	public boolean areAgentsShown()
	{
		return showAgents.isSelected();
	}

	public void environmentGenerated(boolean generated)
	{
		startExercise.setEnabled(generated);
		environment.setEnabled(false);
	}
	public void exerciseGenerated(boolean generated)
	{
		environment.setEnabled(generated);
	}
	public void exerciseStarted(boolean started)
	{
		environment.setEnabled(started);
		if (started)
		{
			showInterface.setSelected(InvestigationMenu.INITIAL_INTERFACE);
			showRewards.setSelected(InvestigationMenu.INITIAL_REWARDS);
			showAgents.setSelected(InvestigationMenu.INITIAL_AGENTS);
		}
	}
	public void exerciseFinished(boolean finished)
	{
		environment.setEnabled(finished);
	}


	public InvestigationMenu(InvestigationTestInterface frame)
	{
		super();
		this.frame = frame;
		initialize();
	}
	private void initialize()
	{
		add(getFile());
		add(getEnvironment());
	}

	private JMenu getFile()
	{
		if (file == null)
		{
			file = new JMenu("File");
			file.setMnemonic('F');

			file.add(getGenerateEnvironment());
			file.add(getLoadEnvironment());
			file.add(getSaveEnvironment());
			file.addSeparator();
			file.add(getStartExercise());
			file.addSeparator();
			file.add(getExit());
		}
		return file;
	}
	private JMenuItem getGenerateEnvironment()
	{
		if (generateEnvironment == null)
		{
			generateEnvironment = new JMenuItem("Generate Environment");
			generateEnvironment.setMnemonic('G');
			generateEnvironment.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					generateEnvironment();
				}
			});
		}
		return generateEnvironment;
	}
	private JMenuItem getLoadEnvironment()
	{
		if (loadEnvironment == null)
		{
			loadEnvironment = new JMenuItem("Load Environment");
			loadEnvironment.setMnemonic('L');
			loadEnvironment.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					loadEnvironment();
				}
			});

			loadEnvironment.setEnabled(false);
		}
		return loadEnvironment;
	}
	private JMenuItem getSaveEnvironment()
	{
		if (saveEnvironment == null)
		{
			saveEnvironment = new JMenuItem("Save Environment");
			saveEnvironment.setMnemonic('S');
			saveEnvironment.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveEnvironment();
				}
			});

			saveEnvironment.setEnabled(false);
		}
		return saveEnvironment;
	}
	private JMenuItem getStartExercise()
	{
		if (startExercise == null)
		{
			startExercise = new JMenuItem("Start Exercise");
			startExercise.setMnemonic('T');
			startExercise.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					startExercise();
				}
			});

			startExercise.setEnabled(false);
		}
		return startExercise;
	}
	private JMenuItem getExit()
	{
		if (exit == null)
		{
			exit = new JMenuItem("Exit");
			exit.setMnemonic('E');
			exit.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					exit();
				}
			});
		}
		return exit;
	}

	private JMenu getEnvironment()
	{
		if (environment == null)
		{
			environment = new JMenu("Environment");
			environment.setMnemonic('E');

			environment.add(getShowInterface());
			environment.add(getShowRewards());
			environment.add(getShowAgents());

			environment.setEnabled(false);
		}
		return environment;
	}
	private JMenuItem getShowInterface()
	{
		if (showInterface == null)
		{
			showInterface = new JCheckBoxMenuItem("Show Interface");
			showInterface.setMnemonic('I');
			showInterface.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					showInterface();
				}
			});


			showInterface.setSelected(InvestigationMenu.INITIAL_INTERFACE);
		}
		return showInterface;
	}
	private JMenuItem getShowRewards()
	{
		if (showRewards == null)
		{
			showRewards = new JCheckBoxMenuItem("Show Rewards");
			showRewards.setMnemonic('R');
			showRewards.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					showRewards();
				}
			});

			showRewards.setSelected(InvestigationMenu.INITIAL_REWARDS);
		}
		return showRewards;
	}
	private JMenuItem getShowAgents()
	{
		if (showAgents == null)
		{
			showAgents = new JCheckBoxMenuItem("Show Agents");
			showAgents.setMnemonic('A');
			showAgents.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					showAgents();
				}
			});

			showAgents.setSelected(InvestigationMenu.INITIAL_AGENTS);
		}
		return showAgents;
	}


	private void generateEnvironment()
	{
		frame.generateEnvironment();
		/*LambdaEnvironment environment = InvestigationTest.getTest().getEnvironment();
		if (environment.isInteracting())
			environment.cancelInteraction();
		environment.clear();
		appController.setPanel(new SpaceOptionsPanel(appController));*/
	}
	private void loadEnvironment()
	{
		
	}
	private void saveEnvironment()
	{
		
	}
	private void startExercise()
	{
		frame.startExercise();
		/*Environment_L environment = InvestigationTest.getTest().getEnvironment();
		if (environment.isInteracting())
			environment.cancelInteraction();
		environment.resetSession();
		((InvestigationMenu) appController.getMenu()).environmentStarted(false);
		appController.setPanel(new SessionOptionsPanel(appController));*/
	}
	private void exit()
	{
		frame.exit();
		//appController.exit();
	}

	private void showInterface()
	{
		frame.showInterface();
	}
	private void showRewards()
	{
		frame.showRewards();
	}
	private void showAgents()
	{
		frame.showAgents();
	}
}