package main;


import java.util.Random;

import anynt.environment.LambdaEnvironment;
import anynt.environment.ReactiveLambdaEnvironment;
import anynt.environment.settings.NoShareRewards;
import anynt.environment.settings.RewardsProvider;
import anynt.environment.settings.ShareRewardsBetweenEvaluableAgents;
import anynt.exercise.Exercise;
import anynt.exercise.settings.InteractionType;
import anynt.exercise.settings.NumberOfInteractionsInteractionType;
import anynt.exercise.settings.StepWaiting;
import anynt.exercise.settings.TimeDelayWaiting;
import anynt.object.EvaluableAgent;
import anynt.object.RewardsGeneratorAgent;
import anynt.object.behaviour.AgentBehaviour;
import anynt.object.behaviour.OracleBehaviour;
import anynt.object.behaviour.PatternBehaviour;
import anynt.object.behaviour.QLearningBehaviour;
import anynt.object.behaviour.RandomBehaviour;
import anynt.object.behaviour.TrivialFollowerBehaviour;
import anynt.object.moveController.IAgentMoveController;
import anynt.object.moveController.NumberOfMovementsMoveController;
import anynt.object.moveController.PatternMoveController;
import anynt.space.Space;


@SuppressWarnings("unused")
public class MainPruebas
{
	private Exercise exercise;
	private String file;


	public static void main(String args[])
	{
		Thread.currentThread().setName("Main");
		MainPruebas pruebas = new MainPruebas();
		pruebas.ejecutarExperimentoDefinido();
		pruebas.ejecutarExperimentosGenerado();
		pruebas.ejecutarExperimentosSocial();
		pruebas.ejecutarExperimentoReactivo();
		//pruebas.encontrarParametrosQLearning();
		//----------- Options -----------//
		// 0 - Min Cells
		// 1 - Max Cells
		// 2 - Min Actions
		// 3 - Max Actions
		// 4 - Rewarders Actions per Step
		//     0 - N Movements
		//     1 - Pattern
		// 5 - Rewarders Fixed Actions
		// 6 - Rewarders Behaviour
		//     0 - Random
		//     1 - Pattern
		// 7 - Rewarders Relocation
		// 8 - Agent
		//     0 - Random
		//     1 - TrivialFollower
		//     2 - Oracle
		//     3 - QLearning
		// 9 - Session Interactions
		// 10 - Session Consumption Order
		//     0 - Before
		//     1 - After
		// 11 - Session Reward Decreasing Factor
		// 12 - Session Consume Rewards
		//     0 - Yes
		//     1 - No
		// 13 - Session Share Rewards
		//     0 - No
		//     1 - Between Evaluable Agents
		//----------- Options -----------//
	}


	private void encontrarParametrosQLearning()
	{
		int sesionesPorParametros = 1000;

		double bestReward = Double.NEGATIVE_INFINITY;
		double bestAlfa = 0.0;
		double bestBeta = 0.0;
		double bestGamma = 0.0;
		double bestQ = 0.0;
		double[] qValues = {2.0};
		System.out.println("Empezando");
		{
			for (double alfa = 0.05; alfa <= 1.0; alfa += 0.05)
			{
				System.out.println("Alfa: " + alfa);
				//for (double beta = 0.05; beta <= 1.0; beta += 0.10)
				{
					double beta = 0.0;//
					System.out.println("Beta: " + beta);
					for (double gamma = 0; gamma < 1.0; gamma += 0.05)
					{
						System.out.println("Gamma: " + gamma);
						for (int Qindex = 0; Qindex < qValues.length; Qindex++)
						{
							double totalReward = 0.0;
							Object[] options = {3, 9, 2, 9, 1, 2, 1, 0, 3, 10000, 1, 2, 0, 0, alfa, beta, gamma, qValues[Qindex]};
							for (int index = 0; index < sesionesPorParametros; index++)
							{
								switch (new Random().nextInt(4))
								{
								case 0:
									definirEntornoDefinido(3);
									break;
								case 1:
									definirEntornoGenerado(options);
									break;
								case 2:
									definirEntornoSocial(options);
									break;
								case 3:
									definirEntornoReactivo(options);
									break;
								}
								lanzarExperimento();
								totalReward += exercise.getEvaluableAgent().totalReward();
							}
							totalReward /= (double) sesionesPorParametros;
							if (totalReward > bestReward)
							{
								bestReward = totalReward;
								bestAlfa = alfa;
								bestBeta = beta;
								bestGamma = gamma;
								bestQ = qValues[Qindex];
							}
						}
					}
				}
			}
		}
		System.out.println("Terminado");
		System.out.println("Mejores resultados:");
		System.out.println("Reward: " + bestReward);
		System.out.println("Alfa: " + bestAlfa);
		System.out.println("Beta: " + bestBeta);
		System.out.println("Gamma: " + bestGamma);
		System.out.println("Q0: " + bestQ);
	}


	private void ejecutarExperimentoDefinido()
	{
		file = files.ExperimentsFile.FILE_PATH_DEFINED;

		definirEntornoDefinido(0);
		lanzarExperimento();
		exportExperiments();
		definirEntornoDefinido(1);
		lanzarExperimento();
		exportExperiments();
		definirEntornoDefinido(2);
		lanzarExperimento();
		exportExperiments();
		definirEntornoDefinido(3);
		lanzarExperimento();
		exportExperiments();
	}
	private void ejecutarExperimentosGenerado()
	{
		file = files.ExperimentsFile.FILE_PATH_GENERATED;

		int[] celdas = {3, 6, 9};
		int[] agents = {0, 1, 2, 3};
		Object[] decreaseFactor = {Integer.MAX_VALUE};
		System.out.println("Empezando");
		for (int celdasIndex = 0; celdasIndex < celdas.length; celdasIndex++)
		{
			System.out.println("Celda: " + celdas[celdasIndex]);
			for (int agentsIndex = 0; agentsIndex < agents.length; agentsIndex++)
			{
				System.out.println("Agente: " + agents[agentsIndex]);
				for (int decreaseFactorIndex = 0; decreaseFactorIndex < decreaseFactor.length; decreaseFactorIndex++)
				{
					// Definir el entorno
					for (int index = 0; index < 100; index++)
					{
						Object[] options = {celdas[celdasIndex], celdas[celdasIndex], 2, 9, 0, 1, 1, 0, agents[agentsIndex], 10000, 1, decreaseFactor[decreaseFactorIndex], 0, 0, 0.05, 0.0, 0.35, 2.0};
						definirEntornoGenerado(options);
						lanzarExperimento();
						exportExperiments();
					}
				}
			}
		}
	}
	private void ejecutarExperimentosSocial()
	{
		file = files.ExperimentsFile.FILE_PATH_SOCIAL;

		int[] celdas = {3, 6, 9};
		int[] agents = {3};
		Object[] decreaseFactor = {Integer.MAX_VALUE};
		System.out.println("Empezando");
		for (int celdasIndex = 0; celdasIndex < celdas.length; celdasIndex++)
		{
			System.out.println("Celda: " + celdas[celdasIndex]);
			for (int agentsIndex = 0; agentsIndex < agents.length; agentsIndex++)
			{
				System.out.println("Agente: " + agents[agentsIndex]);
				for (int decreaseFactorIndex = 0; decreaseFactorIndex < decreaseFactor.length; decreaseFactorIndex++)
				{
					// Definir el entorno
					for (int index = 0; index < 100; index++)
					{
						Object[] options = {celdas[celdasIndex], celdas[celdasIndex], 2, 9, 0, 1, 1, 0, agents[agentsIndex], 10000, 1, decreaseFactor[decreaseFactorIndex], 0, 0, 0.05, 0.0, 0.35, 2.0};
						definirEntornoSocial(options);
						lanzarExperimento();
						exportExperiments();
					}
				}
			}
		}
	}
	private void ejecutarExperimentoReactivo()
	{
		file = files.ExperimentsFile.FILE_PATH_REACTIVE;

		int[] celdas = {3, 6, 9};
		int[] agents = {0, 1, 2, 3};
		Object[] decreaseFactor = {2};
		System.out.println("Empezando");
		for (int celdasIndex = 0; celdasIndex < celdas.length; celdasIndex++)
		{
			System.out.println("Celda: " + celdas[celdasIndex]);
			for (int agentsIndex = 0; agentsIndex < agents.length; agentsIndex++)
			{
				System.out.println("Agente: " + agents[agentsIndex]);
				for (int decreaseFactorIndex = 0; decreaseFactorIndex < decreaseFactor.length; decreaseFactorIndex++)
				{
					// Definir el entorno
					for (int index = 0; index < 100; index++)
					{
						Object[] options = {celdas[celdasIndex], celdas[celdasIndex], 2, 9, 0, 1, 1, 0, agents[agentsIndex], 10000, 1, decreaseFactor[decreaseFactorIndex], 0, 0, 0.05, 0.0, 0.35, 2.0};
						definirEntornoReactivo(options);
						lanzarExperimento();
						exportExperiments();
					}
				}
			}
		}
	}

	private void definirEntornoDefinido(int agente)
	{
		exercise = new Exercise();

		String spaceDescription = "12+3------|12+++++3-----|1-2------3++|1-----2++++++3-|12+3++++++|1-----23-------|1++++++2-------3++|1----2+++3+";
		String rewardersPattern = "203210200";

		LambdaEnvironment environment = new LambdaEnvironment();
		Space space = new Space();
		space.constructSpace(spaceDescription);
		environment.setSpace(space);

		// Prepare Rewards Generators Agents
		RewardsGeneratorAgent goodAgent = environment.getGoodAgent();
		RewardsGeneratorAgent evilAgent = environment.getEvilAgent();
		IAgentMoveController rewardsGeneratorAgentMoveController = new NumberOfMovementsMoveController(1);//new PatternMoveController();
		AgentBehaviour goodAgentBehaviour = new PatternBehaviour(rewardersPattern);
		AgentBehaviour evilAgentBehaviour = goodAgentBehaviour.clone();
		evilAgentBehaviour.setAgent(evilAgent);

		goodAgent.setKnownActions(environment.getActions());
		goodAgent.setMaximumObservationsSaved(1);
		goodAgent.setMoveController(rewardsGeneratorAgentMoveController);
		goodAgent.setBehaviour(goodAgentBehaviour);
		evilAgent.setKnownActions(environment.getActions());
		evilAgent.setMaximumObservationsSaved(1);
		evilAgent.setMoveController(rewardsGeneratorAgentMoveController);
		evilAgent.setBehaviour(evilAgentBehaviour);

		// Prepare Evaluable Agent
		EvaluableAgent agent = new EvaluableAgent("Javier");
		IAgentMoveController evaluableAgentMoveController = new NumberOfMovementsMoveController();
		agent.setKnownActions(environment.getActions());
		agent.setMoveController(evaluableAgentMoveController);
		agent.setMaximumObservationsSaved(5);
//		agent.SetMinimumTime(1);
		agent.setMaximumTimeToAct(5);
		AgentBehaviour behaviour = null;
		switch (agente)
		{
		case 0:
			behaviour = new RandomBehaviour();
			break;
		case 1:
			behaviour = new TrivialFollowerBehaviour(goodAgent, evilAgent);
			break;
		case 2:
			behaviour = new OracleBehaviour(environment);
			break;
		case 3:
			behaviour = new QLearningBehaviour(environment.getAgents(), 0.05, 0.0, 0.35, 2.0);
			break;
		}
		agent.setBehaviour(behaviour);

		environment.addAgent(agent);
		exercise.setEvaluableAgent(agent);
		InteractionType interactionType = new NumberOfInteractionsInteractionType(10000);
		exercise.setInteractionType(interactionType);
		RewardsProvider rewardsSharing = new NoShareRewards();
		environment.setRewardsProvider(rewardsSharing);
		StepWaiting stepWaiting = new TimeDelayWaiting();
		exercise.setStepWaiting(stepWaiting);
		environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(false);
		environment.setDecreaseFactor(Integer.MAX_VALUE);
		environment.setInteractionsToRelocateRewarders(0);
		exercise.setEnvironment(environment);
	}
	private void definirEntornoGenerado(Object[] options)
	{
		exercise = new Exercise();

		LambdaEnvironment environment = new LambdaEnvironment();
		Space space = new Space();

		// Prepare the Space
		space.setCellsBounds((Integer) options[0], (Integer) options[1]);
		space.setActionsBounds((Integer) options[2], (Integer) options[3]);
		do
			space.generateSpace();
		while (!space.isStrongConnected());
		environment.setSpace(space);
		//this.environment.InterpretSpace("1+2+++3|1+2-3---|1++2-3|1---2++3+|1+++2--3-|1--2+3|1-2+3+++|1-2---3");//8
		////////////////////////////////

		// Prepare Rewards Generators Agents
		RewardsGeneratorAgent goodAgent = environment.getGoodAgent();
		RewardsGeneratorAgent evilAgent = environment.getEvilAgent();
		IAgentMoveController rewardsGeneratorAgentMoveController = null;
		switch ((Integer) options[4])
		{
		case 0:
			rewardsGeneratorAgentMoveController = new NumberOfMovementsMoveController();
			((NumberOfMovementsMoveController) rewardsGeneratorAgentMoveController).setNumberOfMovements((Integer) options[5]);
			break;
		case 1:
			rewardsGeneratorAgentMoveController = new PatternMoveController();
			break;
		}
		AgentBehaviour goodAgentBehaviour = null;
		AgentBehaviour evilAgentBehaviour = null;
		switch ((Integer) options[6])
		{
		case 0:
			goodAgentBehaviour = new RandomBehaviour();
			break;
		case 1:
			goodAgentBehaviour = new PatternBehaviour(PatternBehaviour.generatePattern(environment.getActions(), 100));
			break;
		}
		evilAgentBehaviour = goodAgentBehaviour.clone();
		evilAgentBehaviour.setAgent(evilAgent);

		goodAgent.setKnownActions(environment.getActions());
		goodAgent.setMaximumObservationsSaved(1);
		goodAgent.setMoveController(rewardsGeneratorAgentMoveController);
		goodAgent.setBehaviour(goodAgentBehaviour);
		evilAgent.setKnownActions(environment.getActions());
		evilAgent.setMaximumObservationsSaved(1);
		evilAgent.setMoveController(rewardsGeneratorAgentMoveController);
		evilAgent.setBehaviour(evilAgentBehaviour);

		// Introducimos los agentes a evaluar
		IAgentMoveController evaluableAgentMoveController = new NumberOfMovementsMoveController();
		EvaluableAgent agent = null;
		switch ((Integer) options[8])
		{
		case 0:
			agent = new EvaluableAgent("Aleatorio");
			agent.setBehaviour(new RandomBehaviour());
			break;
		case 1:
			agent = new EvaluableAgent("Trivial Follower");
			agent.setBehaviour(new TrivialFollowerBehaviour(goodAgent, evilAgent));
			break;
		case 2:
			agent = new EvaluableAgent("Oracle");
			agent.setBehaviour(new OracleBehaviour(environment));
			break;
		case 3:
			agent = new EvaluableAgent("Q-Learning");
			double alfa = (Double) options[14];
			double beta = (Double) options[15];
			double gamma = (Double) options[16];
			double q0 = (Double) options[17];
			agent.setBehaviour(new QLearningBehaviour(environment.getAgents(), alfa, beta, gamma, q0));
			break;
		}
		agent.setKnownActions(environment.getActions());
		agent.setMoveController(evaluableAgentMoveController);
		agent.setMaximumObservationsSaved(5);
//		agent.setMinimumTime(1);
		agent.setMaximumTimeToAct(5);

		environment.addAgent(agent);
		exercise.setEvaluableAgent(agent);
		/////////////////////////////////////

		int[] interacciones = {/*5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, */(Integer) options[9]};
		double[][] rewards = new double[environment.getAgents().size() - 2][interacciones.length];
		environment.setInteractionsToRelocateRewarders((Integer) options[7]);
		InteractionType interactionType = new NumberOfInteractionsInteractionType(maximo(interacciones));
		//InteractionType interactionType = new TimeLimitInteractionType(interacciones_tiempo[j]);
		exercise.setInteractionType(interactionType);
		RewardsProvider rewardsSharing = null;
		switch ((Integer) options[13])
		{
		case 0:
			rewardsSharing = new NoShareRewards();
			break;
		case 1:
			rewardsSharing = new ShareRewardsBetweenEvaluableAgents();
			break;
		}
		environment.setRewardsProvider(rewardsSharing);
		StepWaiting stepWaiting = new TimeDelayWaiting();
		//((TimeDelayWaiting) stepWaiting).SetTimeDelay(1);
		exercise.setStepWaiting(stepWaiting);
		switch ((Integer) options[10])
		{
		case 0:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(true);
			break;
		case 1:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(false);
			break;
		}
		environment.setDecreaseFactor((Integer) options[11]);
		switch ((Integer) options[12])
		{
		case 0:
			environment.setEvaluableAgentConsumeRewards(true);
			break;
		case 1:
			environment.setEvaluableAgentConsumeRewards(false);
			break;
		}
		exercise.setEnvironment(environment);
	}
	private void definirEntornoSocial(Object[] options)
	{
		exercise = new Exercise();

		LambdaEnvironment environment = new LambdaEnvironment();
		Space space = new Space();

		// Prepare the Space
		space.setCellsBounds((Integer) options[0], (Integer) options[1]);
		space.setActionsBounds((Integer) options[2], (Integer) options[3]);
		do
			space.generateSpace();
		while (!space.isStrongConnected());
		environment.setSpace(space);
		////////////////////////////////

		// Prepare Rewards Generators Agents
		RewardsGeneratorAgent goodAgent = environment.getGoodAgent();
		RewardsGeneratorAgent evilAgent = environment.getEvilAgent();
		IAgentMoveController rewardsGeneratorAgentMoveController = null;
		switch ((Integer) options[4])
		{
		case 0:
			rewardsGeneratorAgentMoveController = new NumberOfMovementsMoveController();
			((NumberOfMovementsMoveController) rewardsGeneratorAgentMoveController).setNumberOfMovements((Integer) options[5]);
			break;
		case 1:
			rewardsGeneratorAgentMoveController = new PatternMoveController();
			break;
		}
		AgentBehaviour goodAgentBehaviour = null;
		AgentBehaviour evilAgentBehaviour = null;
		switch ((Integer) options[6])
		{
		case 0:
			goodAgentBehaviour = new RandomBehaviour();
			break;
		case 1:
			goodAgentBehaviour = new PatternBehaviour(PatternBehaviour.generatePattern(environment.getActions(), 100));
			break;
		}
		evilAgentBehaviour = goodAgentBehaviour.clone();
		evilAgentBehaviour.setAgent(evilAgent);

		goodAgent.setKnownActions(environment.getActions());
		goodAgent.setMaximumObservationsSaved(1);
		goodAgent.setMoveController(rewardsGeneratorAgentMoveController);
		goodAgent.setBehaviour(goodAgentBehaviour);
		evilAgent.setKnownActions(environment.getActions());
		evilAgent.setMaximumObservationsSaved(1);
		evilAgent.setMoveController(rewardsGeneratorAgentMoveController);
		evilAgent.setBehaviour(evilAgentBehaviour);

		// Introducimos los agentes a evaluar
		IAgentMoveController evaluableAgentMoveController = new NumberOfMovementsMoveController();
		EvaluableAgent agent = null;
		switch ((Integer) options[8])
		{
		case 0:
			agent = new EvaluableAgent("Aleatorio");
			agent.setBehaviour(new RandomBehaviour());
			break;
		case 1:
			agent = new EvaluableAgent("Trivial Follower");
			agent.setBehaviour(new TrivialFollowerBehaviour(goodAgent, evilAgent));
			break;
		case 2:
			agent = new EvaluableAgent("Oracle");
			agent.setBehaviour(new OracleBehaviour(environment));
			break;
		case 3:
			agent = new EvaluableAgent("Q-Learning");
			double alfa = (Double) options[14];
			double beta = (Double) options[15];
			double gamma = (Double) options[16];
			double q0 = (Double) options[17];
			agent.setBehaviour(new QLearningBehaviour(environment.getAgents(), alfa, beta, gamma, q0));
			break;
		}
		agent.setKnownActions(environment.getActions());
		agent.setMoveController(evaluableAgentMoveController);
		agent.setMaximumObservationsSaved(5);
//		agent.setMinimumTime(1);
		agent.setMaximumTimeToAct(5);

		environment.addAgent(agent);
		exercise.setEvaluableAgent(agent);
		/////////////////////////////////////

		int[] interacciones = {/*5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, */(Integer) options[9]};
		environment.setInteractionsToRelocateRewarders((Integer) options[7]);
		InteractionType interactionType = new NumberOfInteractionsInteractionType(maximo(interacciones));
		//InteractionType interactionType = new TimeLimitInteractionType(interacciones_tiempo[j]);
		exercise.setInteractionType(interactionType);
		RewardsProvider rewardsSharing = null;
		switch ((Integer) options[13])
		{
		case 0:
			rewardsSharing = new NoShareRewards();
			break;
		case 1:
			rewardsSharing = new ShareRewardsBetweenEvaluableAgents();
			break;
		}
		environment.setRewardsProvider(rewardsSharing);
		StepWaiting stepWaiting = new TimeDelayWaiting();
		//((TimeDelayWaiting) stepWaiting).SetTimeDelay(1);
		exercise.setStepWaiting(stepWaiting);
		switch ((Integer) options[10])
		{
		case 0:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(true);
			break;
		case 1:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(false);
			break;
		}
		environment.setDecreaseFactor((Integer) options[11]);
		switch ((Integer) options[12])
		{
		case 0:
			environment.setEvaluableAgentConsumeRewards(true);
			break;
		case 1:
			environment.setEvaluableAgentConsumeRewards(false);
			break;
		}

		EvaluableAgent agent2 = new EvaluableAgent("Aleatorio");
		agent2.setBehaviour(new RandomBehaviour());
		agent2.setKnownActions(environment.getActions());
		agent2.setMoveController(evaluableAgentMoveController);
		agent2.setMaximumObservationsSaved(5);
//		agent2.setMinimumTime(1);
		agent2.setMaximumTimeToAct(5);
		environment.addAgent(agent2);
		EvaluableAgent agent3 = new EvaluableAgent("Trivial Follower");
		agent3.setKnownActions(environment.getActions());
		agent3.setBehaviour(new TrivialFollowerBehaviour(goodAgent, evilAgent));
		agent3.setMoveController(evaluableAgentMoveController);
		agent3.setMaximumObservationsSaved(5);
//		agent3.setMinimumTime(1);
		agent3.setMaximumTimeToAct(5);
		environment.addAgent(agent3);
		EvaluableAgent agent4 = new EvaluableAgent("Oracle");
		agent4.setKnownActions(environment.getActions());
		agent4.setBehaviour(new OracleBehaviour(environment));
		agent4.setMoveController(evaluableAgentMoveController);
		agent4.setMaximumObservationsSaved(5);
//		agent4.setMinimumTime(1);
		agent4.setMaximumTimeToAct(5);
		environment.addAgent(agent4);
		/*EvaluableAgent agent5 = new EvaluableAgent("Q-Learning", this.environment);
		double alfa = 0.75;
		double beta = 0.15;
		double gamma = 0.25;
		double q0 = 2.0;
		agent5.setBehaviour(new QLearningBehaviour(agent5, this.environment.getAgents(), alfa, beta, gamma, q0));
		agent5.setMoveController(evaluableAgentMoveController);
		agent5.setNObservationsSaved(5);
		agent5.setMinimumTime(1);
		agent5.setMaximumTime(5);
		this.environment.addAgent(agent5);*/
		exercise.setEnvironment(environment);
	}
	private void definirEntornoReactivo(Object[] options)
	{
		exercise = new Exercise();

		LambdaEnvironment environment = new ReactiveLambdaEnvironment();
		Space space = new Space();

		// Prepare the Space
		space.setCellsBounds((Integer) options[0], (Integer) options[1]);
		space.setActionsBounds((Integer) options[2], (Integer) options[3]);
		do
			space.generateSpace();
		while (!space.isStrongConnected());
		environment.setSpace(space);
		////////////////////////////////

		// Prepare Rewards Generators Agents
		RewardsGeneratorAgent goodAgent = environment.getGoodAgent();
		RewardsGeneratorAgent evilAgent = environment.getEvilAgent();
		IAgentMoveController rewardsGeneratorAgentMoveController = null;
		switch ((Integer) options[4])
		{
		case 0:
			rewardsGeneratorAgentMoveController = new NumberOfMovementsMoveController();
			((NumberOfMovementsMoveController) rewardsGeneratorAgentMoveController).setNumberOfMovements((Integer) options[5]);
			break;
		case 1:
			rewardsGeneratorAgentMoveController = new PatternMoveController();
			break;
		}
		AgentBehaviour goodAgentBehaviour = null;
		AgentBehaviour evilAgentBehaviour = null;
		switch ((Integer) options[6])
		{
		case 0:
			goodAgentBehaviour = new RandomBehaviour();
			break;
		case 1:
			goodAgentBehaviour = new PatternBehaviour(PatternBehaviour.generatePattern(environment.getActions(), 100));
			break;
		}
		evilAgentBehaviour = goodAgentBehaviour.clone();
		evilAgentBehaviour.setAgent(evilAgent);

		goodAgent.setKnownActions(environment.getActions());
		goodAgent.setMaximumObservationsSaved(1);
		goodAgent.setMoveController(rewardsGeneratorAgentMoveController);
		goodAgent.setBehaviour(goodAgentBehaviour);
		evilAgent.setKnownActions(environment.getActions());
		evilAgent.setMaximumObservationsSaved(1);
		evilAgent.setMoveController(rewardsGeneratorAgentMoveController);
		evilAgent.setBehaviour(evilAgentBehaviour);

		// Introducimos los agentes a evaluar
		IAgentMoveController evaluableAgentMoveController = new NumberOfMovementsMoveController();
		EvaluableAgent agent = null;
		switch ((Integer) options[8])
		{
		case 0:
			agent = new EvaluableAgent("Aleatorio");
			agent.setBehaviour(new RandomBehaviour());
			break;
		case 1:
			agent = new EvaluableAgent("Trivial Follower");
			agent.setBehaviour(new TrivialFollowerBehaviour(goodAgent, evilAgent));
			break;
		case 2:
			agent = new EvaluableAgent("Oracle");
			agent.setBehaviour(new OracleBehaviour(environment));
			break;
		case 3:
			agent = new EvaluableAgent("Q-Learning");
			double alfa = (Double) options[14];
			double beta = (Double) options[15];
			double gamma = (Double) options[16];
			double q0 = (Double) options[17];
			agent.setBehaviour(new QLearningBehaviour(environment.getAgents(), alfa, beta, gamma, q0));
			break;
		}
		agent.setKnownActions(environment.getActions());
		agent.setMoveController(evaluableAgentMoveController);
		agent.setMaximumObservationsSaved(5);
//		agent.setMinimumTime(1);
		agent.setMaximumTimeToAct(5);

		environment.addAgent(agent);
		exercise.setEvaluableAgent(agent);
		/////////////////////////////////////

		int[] interacciones = {/*5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, */(Integer) options[9]};
		environment.setInteractionsToRelocateRewarders((Integer) options[7]);
		InteractionType interactionType = new NumberOfInteractionsInteractionType(maximo(interacciones));
		//InteractionType interactionType = new TimeLimitInteractionType(interacciones_tiempo[j]);
		exercise.setInteractionType(interactionType);
		RewardsProvider rewardsSharing = null;
		switch ((Integer) options[13])
		{
		case 0:
			rewardsSharing = new NoShareRewards();
			break;
		case 1:
			rewardsSharing = new ShareRewardsBetweenEvaluableAgents();
			break;
		}
		environment.setRewardsProvider(rewardsSharing);
		StepWaiting stepWaiting = new TimeDelayWaiting();
		//((TimeDelayWaiting) stepWaiting).SetTimeDelay(1);
		exercise.setStepWaiting(stepWaiting);
		switch ((Integer) options[10])
		{
		case 0:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(true);
			break;
		case 1:
			environment.setRewardsConsumedBeforeRewardersUpdateCellRewards(false);
			break;
		}
		environment.setDecreaseFactor((Integer) options[11]);
		switch ((Integer) options[12])
		{
		case 0:
			environment.setEvaluableAgentConsumeRewards(true);
			break;
		case 1:
			environment.setEvaluableAgentConsumeRewards(false);
			break;
		}
		exercise.setEnvironment(environment);
	}

	private void lanzarExperimento()
	{
		int[] interacciones = {10000};
		double[][] rewards = new double[exercise.getEnvironment().getAgents().size() - 2][interacciones.length];

		exercise.checkConsistency();
		exercise.interact();
		for (int indiceAgente = 2; indiceAgente < exercise.getEnvironment().getAgents().size(); indiceAgente++)
		{
			EvaluableAgent agente = (EvaluableAgent) exercise.getEnvironment().getAgents().get(indiceAgente);
			for (int index = 0; index < interacciones.length; index++)
				rewards[indiceAgente - 2][index] = agente.totalReward(interacciones[index]);
		}
	}




	private int maximo(int[] vector)
	{
		int max = vector[0];
		for (int indice = 1; indice < vector.length; indice++)
			if (vector[indice] > max)
				max = vector[indice];
		return max;
	}
	private void printResultados(int[] interacciones, double[][] recompensas)
	{
		String[] agentes = new String[exercise.getEnvironment().getAgents().size() - 2];
		int maximaLongitudAgente = 0;
		for (int indice = 0; indice < agentes.length; indice++)
		{
			agentes[indice] = exercise.getEnvironment().getAgents().get(indice + 2).getName();
			if (agentes[indice].length() > maximaLongitudAgente)
				maximaLongitudAgente = agentes[indice].length();
		}

		System.out.println("Resultado de los experimentos");
		System.out.println("-----------------------------");
		System.out.println("");

		for (int indiceAgente = 0; indiceAgente < exercise.getEnvironment().getAgents().size() - 2; indiceAgente++)
		{
			System.out.println("Agente: " + agentes[indiceAgente]);
			System.out.println("------------------------------------");
			for (int index = 0; index < interacciones.length; index++)
				System.out.println(interacciones[index] + " interacciones: " + recompensas[indiceAgente][index]);
			System.out.println("");
		}
	}
	private void exportExperiments()
	{
		boolean experimentsExportedSuccessfully = files.ExperimentsFile.WriteExperiment(exercise, file);
		/*if (experimentsExportedSuccessfully == true)
			System.out.println("\nExperiments exported successfully");
		else
			System.out.println("\nSystem crashed while trying to export the experiments");*/
	}
}